#include "stc8h.h"
#include "intrins.h"

#include "wexcube.h"


#define MAIN_Fosc       	11059200L   // 定义主时钟
#define SERVO_PIN 			P15      	// P1.5引脚舵机控制

#define BUTTON_ON_ID		1			// 控制开的控件ID
#define BUTTON_OFF_ID		2			// 控制关的控件ID

#define DEFAULT_ANGLE		95			// 默认舵机角度
#define ON_ANGLE			124			// 开的角度
#define OFF_ANGLE			64			// 关的角度

wex_u16_t pulseWidth = 0;


void timer0_init(void);
void set_angle(wex_u8_t angle);
void delay_ms(wex_u8_t ms);


void main()
{
    P1M0 = 0x00;
    P1M1 = 0x00;

	timer0_init();
	wex_init();
    wex_start();

    EA = 1;
	
	set_angle(DEFAULT_ANGLE);

    while(1)
    {
		const t_sWexCmd *psWexCmd = wex_process();
        switch (psWexCmd->eCmdType)
        {
            case eWexCmd_Event:
			{
				switch (psWexCmd->ucCtrlId)
				{
					case BUTTON_ON_ID:
					{
						if (psWexCmd->ucValue)
						{
							set_angle(ON_ANGLE);
							delay_ms(1000);
							set_angle(DEFAULT_ANGLE);
						}
					}
					break;
					
					case BUTTON_OFF_ID:
					{
						if (psWexCmd->ucValue)
						{
							set_angle(OFF_ANGLE);
							delay_ms(1000);
							set_angle(DEFAULT_ANGLE);
						}
					}
					break;
					
					default:
					break;
				}
			}
            break;

            default:
            break;
        }
		
		delay_ms(10);
    }
}


/* 定时器配置修改说明：
   在11.0592MHz主频、12T模式下：
   定时器时钟 = 11.0592MHz / 12 = 921.6kHz
   定时周期 = 1/921.6kHz ≈ 1.085μs
   需要50μs中断：
   计数值 = 50μs / 1.085μs ≈ 46 → 初值 = 65536 - 46 = 65490 → 0xFFD2
*/
void timer0_init(void)
{
    TMOD &= 0xF0;
    TMOD |= 0x01;
    TL0 = 0xD2;           // 修改初值为0xD2（原0x9C）
    TH0 = 0xFF;
    ET0 = 1;
    TR0 = 1;
}

// 中断服务函数保持不变
void Timer0_ISR() interrupt 1
{
    static unsigned int count = 0;
    
    TL0 = 0xD2;           // 同步修改重载值
    TH0 = 0xFF;
    
    count++;
    
    if(count <= pulseWidth) {
        SERVO_PIN = 1;
    } else {
        SERVO_PIN = 0;
    }
    
    if(count >= 400) {     // 保持20ms周期（400*50μs）
        count = 0;
    }
}

// 角度转换函数保持不变
void set_angle(unsigned char angle)
{
    pulseWidth = 10 + (unsigned int)(angle * 40) / 180;
}



//========================================================================
// 函数: void delay_ms(u8 ms)
// 描述: 延时函数。
// 参数: ms,要延时的ms数, 这里只支持1~255ms. 自动适应主时钟.
// 返回: none.
// 版本: VER1.0
// 日期: 2021-3-9
// 备注: 
//========================================================================
void delay_ms(wex_u8_t ms)
{
     wex_u16_t i;
     do{
          i = MAIN_Fosc / 10000;
          while(--i);   //10T per loop
     }while(--ms);
}


